<?php
/**
 * 开放银行平台SDK工具类
 *
 * @author xiezz, zengminghua
 * @version 1.0.0
 */

// 以下常量为SDK出现错误时，返回的字符串，可以根据需要自己设置（注意不是服务端返回的）
// 通讯失败时返回的报文
define('TXN_ERROR_RESULT', "{\"code\":\"OPEN25801\",\"msg\":\"通讯错误或超时，交易未决\",\"traceId\":\"OPEN-00-LOCAL-801\"}");
// 参数格式无效时返回的报文
define('INVALID_PARAM_RESULT', "{\"code\":\"OPEN25802\",\"msg\":\"参数格式无效，请检查\",\"traceId\":\"OPEN-00-LOCAL-802\"}");
// 系统异常时返回的报文
define('SYS_ERROR_RESULT', "{\"code\":\"OPEN25803\",\"msg\":\"未知错误，请检查是否为最新版本SDK或是否配置错误\",\"traceId\":\"OPEN-00-LOCAL-803\"}");
// 文件下载成功
define('FILE_SUCCESS_RESULT', "{\"code\":\"OPEN25800\",\"msg\":\"下载成功\",\"traceId\":\"OPEN-00-LOCAL-800\"}");
// 文件写入失败
define('FILE_ERROR_RESULT', "{\"code\":\"OPEN25805\",\"msg\":\"文件读取或写入失败\",\"traceId\":\"OPEN-00-LOCAL-805\"}");


class OpenUtil
{

    /**
     * 获取当前系统日期
     * @return string 当前日期，20170101格式
     */
    public static function getDate()
    {

        return date('Ymd');
    }

    /**
     * 获取当前系统日期时间
     * @return string 当前日期时间，20180101010203格式
     */
    public static function getDateTime()
    {

        return date('YmdHis');
    }

    /**
     * HTTP请求通讯器
     * @param string $url 通讯URL
     * @param string $method HTTP方法，可为GET/POST，默认GET
     * @param array $url_param_array URL上参数列表
     * @param array $body_param_array body中参数列表
     * @param array $header_param_array HTTP头部参数列表
     * @param boolean $skip_ssl_verify 是否需要验证服务器证书，true-验证，false-不验证
     * @param string $proxy_ip 代理服务器IP
     * @param string $proxy_port 代理服务器端口
     * @param string $save_file_name 文件保存路径
     * @return string 服务端返回的结果
     */
    public static function getHttpResponse($url, $method, $url_param_array, $body_param_array, $header_param_array, $skip_ssl_verify = false, $proxy_ip = null, $proxy_port = null, $save_file_name = null, $use_multipart = false)
    {
        if (is_array($url_param_array) && count($url_param_array) > 0)
            $url .= ('?' . http_build_query($url_param_array));

        $curl = curl_init($url);
        if (!$skip_ssl_verify) {
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, true);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 2);
            // 信任CA证书相对路径，如果不是在这里，需要修改该变更
            curl_setopt($curl, CURLOPT_CAINFO, dirname(__FILE__) . '/ca-bundle.crt');
        } else {
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
        }

        if ($proxy_ip && $proxy_port) {
            curl_setopt($curl, CURLOPT_PROXY, $proxy_ip);
            curl_setopt($curl, CURLOPT_PROXYPORT, $proxy_port);
        }

        curl_setopt($curl, CURLOPT_HEADER, 0);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        if (strcmp("POST", $method) == 0) {
            curl_setopt($curl, CURLOPT_POST, true);
            if ($use_multipart) {
                curl_setopt($curl, CURLOPT_POSTFIELDS, $body_param_array);
            } else {
                curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($body_param_array));
            }
        }

        if (is_array($header_param_array) && count($header_param_array) > 0) {
            $header_array = array();
            foreach ($header_param_array as $k => $v) {
                array_push($header_array, $k . ': ' . $v);
            }
            curl_setopt($curl, CURLOPT_HTTPHEADER, $header_array);
        }

        $response = curl_exec($curl);

        // 需要调试时可以去掉以下两行的注释
        // var_dump(curl_error($curl));
        // var_dump(curl_getinfo($curl));

        $header = curl_getinfo($curl);
        curl_close($curl);

        if (!is_int($header['http_code']))
            return TXN_ERROR_RESULT;

        // 文件下载
        if ($save_file_name && $header['content_type'] === 'application/octet-stream') {
            if (false === @file_put_contents($save_file_name, $response))
                return FILE_ERROR_RESULT;
            else
                return FILE_SUCCESS_RESULT;
        } else {
            // 返回json或下载失败
            return $response;
        }
    }

    /**
     * 获取服务器IP地址
     * @return string 服务器IP地址
     */
    public static function getLocalIp()
    {
        if (isset($_ENV["HOSTNAME"]))
            $machineName = $_ENV["HOSTNAME"];
        else if (isset($_ENV["COMPUTERNAME"]))
            $machineName = $_ENV["COMPUTERNAME"];
        else $machineName = '';
        return gethostbyname($machineName);
    }
}
